-- adds a recipe which is unlocked when the given technology is researched
function addTechnologyUnlocksRecipe(technologyName, recipeName)
	local tech = data.raw["technology"][technologyName]
	if tech then
		local recipe = data.raw.recipe[recipeName]
		if recipe == nil then
			error("Technology "..technologyName.." should unlock "..recipeName.." but recipe is not initialized yet")
		else
			data.raw["recipe"][recipeName].enabled = false
			if tech.effects == nil then
				tech.effects = {}
			end
			table.insert(tech.effects, {type = "unlock-recipe", recipe = recipeName })
		end
	else
		error("Technology "..technologyName.." not found when adding recipe "..recipeName..". Did you mean?")
		for name,_ in pairs(data.raw["technology"]) do
			error(" "..name)
		end
	end
end


function make_pins(index)

local pin_input_cc =
      {
        red = {-0.072, 0.4 - (index > 2 and 1.2 or 0)},
        green = {-0.072, 0.25 - (index > 2 and 1.2 or 0)},
      }

local pin_output_cc =
      {
          red = {-0.03, 0.45 - (index > 2 and 1.2 or 0)},
          green = {-0.03, 0.30 - (index > 2 and 1.2 or 0)},
      }


local pins_shift = {0.0, 0.0}
-- {0.0, -0.6+0.2*index}

data:extend({
  	{
		type = "item",
		name = "locc_pini"..index,
		icon = "__ComplexCombinator__/graphics/icons/pin.png",
		flags = {"goes-to-quickbar"},
		subgroup = "circuit-network",
		order = "b[combinators]-l[complex-combinator]",
		place_result = "locc_pini"..index,
		stack_size = 10
	},


	{
		type = "lamp",
		name = "locc_pini"..index,
		icon = "__ComplexCombinator__/graphics/icons/pin.png",
		flags = {"placeable-neutral", "player-creation"},
		minable = {hardness = 0.01, mining_time = 0.01, result = "complex-combinator"},
		max_health = 55,
		corpse = "small-remnants",
		collision_box = {{-0.1, -0.1}, {0.1, 0.1}},
		selection_box = {{-0.1, 0.1 - (index > 2 and 1.2 or 0)}, {0.5, 0.7 - (index > 2 and 1.2 or 0)}},
		vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
		energy_source =
		{
		  type = "electric",
		  usage_priority = "secondary-input"
		},
		energy_usage_per_tick = "1KW",
		light = {intensity = 1, size = 1},
		
		picture_off =
			{
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
			},
		
		
		picture_on =
			{
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
			},
			
		
	
    circuit_wire_connection_point =
    {
      shadow = pin_input_cc,
      wire = pin_input_cc
    },

		circuit_wire_max_distance = 30
	},

	
	
	
	




	
  	{
		type = "item",
		name = "locc_pino"..index,
		icon = "__ComplexCombinator__/graphics/icons/pin.png",
		flags = {"goes-to-quickbar"},
		subgroup = "circuit-network",
		order = "b[combinators]-l[complex-combinator]",
		place_result = "locc_pino"..index,
		stack_size = 10
	},

	{
    name = "locc_pino"..index,
    item_slot_count = 100,
    circuit_wire_max_distance = 30,
	
    type = "constant-combinator",
    icon = "__base__/graphics/icons/constant-combinator.png",
    flags = {"placeable-neutral", "player-creation"},
    minable = {hardness = 0.2, mining_time = 0.5, result = "constant-combinator"},
    max_health = 120,
    corpse = "small-remnants",

	collision_box = {{-0.1, -0.1}, {0.1, 0.1}},
	selection_box = {{-0.5, 0.1 - (index > 2 and 1.2 or 0)}, {0.1, 0.7 - (index > 2 and 1.2 or 0)}},

    sprites =
    {
      north =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
      },
      east =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
       },
      south =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
       },
      west =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
       }
    },

    activity_led_sprites =
    {
      north =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
      },
      east =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
      },
      south =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
      },
      west =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
      }
    },

    activity_led_light =
    {
      intensity = 0.8,
      size = 1,
      color = {r = 1.0, g = 1.0, b = 1.0}
    },

    activity_led_light_offsets =
    {
      {0.296875, -0.40625},
      {0.25, -0.03125},
      {-0.296875, -0.078125},
      {-0.21875, -0.46875}
    },

    circuit_wire_connection_points =
    {
      {
        shadow = pin_output_cc,
        wire = pin_output_cc
      },
      {
        shadow = pin_output_cc,
        wire = pin_output_cc
      },
      {
        shadow = pin_output_cc,
        wire = pin_output_cc
      },
      {
        shadow = pin_output_cc,
        wire = pin_output_cc
      }
    }
  }
})
end

make_pins(1)
make_pins(2)
make_pins(3)
make_pins(4)


data:extend({
  	{
		type = "item",
		name = "complex-combinator",
		icon = "__ComplexCombinator__/graphics/icons/complex-combinator-icon.png",
		flags = {"goes-to-quickbar"},
		subgroup = "circuit-network",
		order = "b[combinators]-l[complex-combinator]",
		place_result = "complex-combinator",
		stack_size = 10
	},
	
	{
		type = "recipe",
		name = "complex-combinator",
		energy_required = 10,
		enabled = false,
		ingredients =
		{
			{"iron-plate", 10},
			{"copper-cable", 20},
			{"electronic-circuit", 20},
		},
		result = "complex-combinator"
	}

	
	
	
	
	
	
	
	
,	{
		type = "lamp",
		name = "complex-combinator",
		icon = "__ComplexCombinator__/graphics/icons/complex-combinator-icon.png",
		flags = {"placeable-neutral", "player-creation"},
		minable = {hardness = 0.01, mining_time = 0.01, result = "complex-combinator"},
		max_health = 55,
		corpse = "small-remnants",
		
		collision_box = {{-1.2, -2.2}, {1.2, 2.2}},
		selection_box = {{-0.7, -2.3}, {0.7, 2.3}},

		vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
		energy_source =
		{
		  type = "electric",
		  usage_priority = "secondary-input"
		},
		energy_usage_per_tick = "1KW",
		light = {intensity = 1, size = 1},
		picture_off =
		{
			filename = "__ComplexCombinator__/graphics/entities/complex-combinator.png",
			x = 0,
			y = 0,
			width = 99,
			height = 148,
			frame_count = 1,
			shift = {0.0, 0.0},
		},
		picture_on =
		{
			filename = "__ComplexCombinator__/graphics/entities/complex-combinator.png",
			x = 0,
			y = 0,
			width = 99,
			height = 148,
			frame_count = 1,
			shift = {0.0, 0.0},
		},
		
	
    circuit_wire_connection_point =
    {
      shadow =
      {
        red = {0.125, 0.5},
        green = {0.875, 0.5},
      },
      wire =
      {
        red = {0.125, 0.5},
        green = {0.875, 0.5},
      }
    },

		circuit_wire_max_distance = 30
	}
	
	
	
	
	

	
	
	
	
	
,  	{
		type = "item",
		name = "locc_pinbd",
		icon = "__ComplexCombinator__/graphics/icons/pin.png",
		flags = {"goes-to-quickbar"},
		subgroup = "circuit-network",
		order = "b[combinators]-l[complex-combinator]",
		place_result = "locc_pinbd",
		stack_size = 10
	}

	
	
	


	
	
	
,	{
    type = "constant-combinator",
    name = "locc_pinbd",
    item_slot_count = 500,
	
	icon = "__ComplexCombinator__/graphics/icons/complex-combinator-icon.png",
    flags = {"placeable-neutral", "player-creation"},
    max_health = 50,
    corpse = "small-remnants",

    collision_box = {{-0.35, -0.35}, {0.35, 0.35}},
    selection_box = {{-0.4, -0.4}, {0.4, 0.4}},

	
    sprites =
    {
      north =
      {
        filename = "__ComplexCombinator__/graphics/empty_1x1.png",
        x = 0,
        y = 0,
        width = 1,
        height = 1,
        frame_count = 1,
        shift = {0.0, 0.0},
      },
      east =
      {
        filename = "__ComplexCombinator__/graphics/empty_1x1.png",
        x = 0,
        y = 0,
        width = 1,
        height = 1,
        frame_count = 1,
        shift = {0.0, 0.0},
      },
      south =
      {
        filename = "__ComplexCombinator__/graphics/empty_1x1.png",
        x = 0,
        y = 0,
        width = 1,
        height = 1,
        frame_count = 1,
        shift = {0.0, 0.0},
      },
      west =
      {
        filename = "__ComplexCombinator__/graphics/empty_1x1.png",
        x = 0,
        y = 0,
        width = 1,
        height = 1,
        frame_count = 1,
        shift = {0.0, 0.0},
      }
    },

    activity_led_sprites =
    {
      north =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
      },
      east =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
      },
      south =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
      },
      west =
      {
			filename = "__ComplexCombinator__/graphics/empty_1x1.png",
			x = 0,
			y = 0,
			width = 1,
			height = 1,
			frame_count = 1,
			shift = {0.0, 0.0},
      }
    },

    activity_led_light =
    {
      intensity = 0.8,
      size = 1,
    },

    activity_led_light_offsets =
    {
      {0.296875, -0.40625},
      {0.25, -0.03125},
      {-0.296875, -0.078125},
      {-0.21875, -0.46875}
    },

    circuit_wire_connection_points =
    {
      {
        shadow =
        {
          red = {0.15625, -0.28125},
          green = {0.65625, -0.25}
        },
        wire =
        {
          red = {-0.28125, -0.5625},
          green = {0.21875, -0.5625},
        }
      },
      {
        shadow =
        {
          red = {0.75, -0.15625},
          green = {0.75, 0.25},
        },
        wire =
        {
          red = {0.46875, -0.5},
          green = {0.46875, -0.09375},
        }
      },
      {
        shadow =
        {
          red = {0.75, 0.5625},
          green = {0.21875, 0.5625}
        },
        wire =
        {
          red = {0.28125, 0.15625},
          green = {-0.21875, 0.15625}
        }
      },
      {
        shadow =
        {
          red = {-0.03125, 0.28125},
          green = {-0.03125, -0.125},
        },
        wire =
        {
          red = {-0.46875, 0},
          green = {-0.46875, -0.40625},
        }
      }
    },

    circuit_wire_max_distance = 7.5
  }

})

data.raw["gui-style"].default["complex_combinator_textbox"] =
    {
      type = "textfield_style",
      parent = "textfield_style",      
	  minimal_width = 80,
      maximal_width = 80,  
	}
	
data.raw["gui-style"].default["complex_combinator_dropdown"] =
    {
      type = "dropdown_style",
      parent = "dropdown_style",      
	  minimal_width = 64,
      maximal_width = 64,  
	}

	
addTechnologyUnlocksRecipe("circuit-network","complex-combinator")